/*
Demonstration of the use of NLMIXED to fit the binomial and beta-binomal
to discrete data using maximum likelihood. Data files can contain just 
the counts (numbers) per sampling unit, labeled y here (not x), or both y 
and n (observations per sampling unit). Total number of sampling units
is N (where lower-case n is individuals in each sampling unit).

If n is constant across sampling units, one can simply define it in a 
data step, such as: "n = 9;". See first example.

It is NOT possible right now to enter the data in frequency format (you must
explicitly give the count for each sampling unit).

Parameters are:
	logit ('linear predictor' or lp), and not p directly; and
	theta (the).
This makes it easy to expand for the inclusion of treatment effects, 
in the framework of generalized linear modeling of data.  For the 
beta-binomial, p is determined from the logit, and also the alpha 
and beta parameters are also displayed (based on p and theta), because
some references use the alpha/beta version of the distribution.
Moreover, the intraclass correlation is also calculated based
on estimated parameters. For the beta-binomial, the theta estimate 
is given under "Parameter Estimates". However, p (and alpha, beta,
& intraclass correlation) are given under "Additional Estimates".

The program uses a dual quasi-Newton optimization algorithm 
by default, which is different from the Newton-Raphson algorithm in 
the BBD program. Results could differ slightly. With NLMIXED, the
optimization algorithm can be changed using "tech=xxxxx" option.

The chi-square goodness of fit test is not done based on observed
and expected frequencies. However, a likelihood-ratio test can
be conducted based on the log-likelihoods for the binomial and
beta-binomial (i.e., does beta-binomial fit better than binomial?).
In the BBD program, only the portion of the log-likelihood 
involving the parameters is displayed. In NLMIXED, the full log-
likelihood is given. The differences in log-likelihoods (binomial and
beta-binomial) are the same for both programs.

Code by: L. V. Madden (Ohio State University). This code is supplied
for the benefit of researchers. There is no guarantee that the 
program will work correctly in all situations.

For more information, contact him at MADDEN.1@osu.edu

*/


*---(1) Tobacco virus data (G1-11);
data tobacco;
input y @@;	*<--Read in data strung out across records (N=75 SUs total here);
n=40;		*<--When n (obs./SU) is always same, one can define it here;
datalines;
 	1    1    1    1    2    2    2    2    2    2    2    2    2    3    3
    3    3    3    3    3    3    4    4    4    4    4    4    4    4    4
    4    4    5    5    5    6    6    6    6    6    6    6    6    6    7
    7    7    7    7    7    7    7    8    8    8    8    8    8    9   10
   10   10   10   10   10   11   11   11   11   12   12   13   14   15   25
;
title 'Fitting distrete distributions to G1-11 tobacco virus example data, ex. 1 (G1-11)';
proc nlmixed data=tobacco ;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
    title2 'Binomial distribution';
							*--Give initial guess of parameters;
    parameters lp = -2.0 ; 	*--Instead of directly estimating p, the LOGIT is directly modeled;
							*--Formulated as a logit linear predictor (lp ), for possible later treatment analysis;
    p=1/(1+exp(-lp));		*--Probability of disease;
    
	model y ~ binomial(n,p);		*--Use the NLMIXED binomial(,) distribution;
	estimate 'p' 1/(1+exp(-lp));	*--Reformulate to get probability estimate (and se);
	
run;

*--The fitting of the beta-binomial using MLE;
proc nlmixed data=tobacco ;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
	title2 'Beta-binomial distribution';
										*--Give initial guess of parameters;
    parameters lp = -2.0 the = 0.1 ; 	*--LOGIT linear predictor (lp) and theta (the);
    p=1/(1+exp(-lp));					*--Get p after first getting the logit;
    
	*--_Following lines give the log-linear formulation for the BBD log-likelihood;
	*--_The combinatorial and the Beta functions are all re-expresed as 
		log-gamma (lgamma) functions;
	
	C1 = (lgamma(n+1) -lgamma(y+1) - lgamma(n-y+1) );
	C2=lgamma((p/the)+((1-p)/the)) + lgamma((p/the)+y) + 
		lgamma(((1-p)/the) + n - y) -
		( lgamma(p/the) + lgamma((1-p)/the) + 
		lgamma((p/the) + ((1-p)/the) + n) );
	ll = C1 + C2;						*--Log-likelihood;

	model y ~ general(ll);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	estimate 'p' 1/(1+exp(-lp));  		*--Get various parameterizations;
	estimate 'alpha' p/the;				
	estimate 'beta' (1-p)/the;
	estimate 'rho (intraclass corr.)' the/(1+the);
run;

data williams;	*---Williams 1975 treated group, toxicological data;
input y n;
datalines;
 12 12
 11 11
 10 10
  9  9
 10 11
  9 10
  9 10
  8  9
  8  9
  4  5
  7  9
  4  7
  5 10
  3  6
  3 10
  0  7
;

title 'Fitting distrete distributions to Williams (1975) treatment group (Ex. 6)';
proc nlmixed data=williams;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
    title2 'Binomial distribution';
    parameters lp = 2.0 ; 	*--Instead of directly estimating p, the LOGIT is directly modeled;
							*--Formulated as a logit linear predictor (lp ), for possible later treatment analysis;
    p=1/(1+exp(-lp));		*--Probability of disease;
    
	model y ~ binomial(n,p);		*--Use the NLMIXED binomial(,) distribution;
	estimate 'p' 1/(1+exp(-lp));	*--Reformulate to get probability estimate (and se);
run;

*--The fitting of the beta-binomial using MLE;
proc nlmixed data=williams;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
	title2 'Beta-binomial distribution';
    parameters lp = -2.0 the = 0.1 ; 	*--LOGIT linear predictor (lp) and theta (the);
								
    p=1/(1+exp(-lp));					*--Get p after first getting the logit;
    
	*--Following lines give the log-linear formulation for the BBD log-likelihood;
	*--The combinatorial and the Beta functions are all re-expresed as lgamma functions;
	
	C1 = (lgamma(n+1) -lgamma(y+1) - lgamma(n-y+1) );
	C2=lgamma((p/the)+((1-p)/the)) + lgamma((p/the)+y) + 
		lgamma(((1-p)/the) + n - y) -
		( lgamma(p/the) + lgamma((1-p)/the) + 
		lgamma((p/the) + ((1-p)/the) + n) );
	ll = C1 + C2;						*--Log-likelihood;

	model y ~ general(ll);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	estimate 'p' 1/(1+exp(-lp));  	*--Get various parameterizations;
	estimate 'alpha' p/the;				
	estimate 'beta' (1-p)/the;
	estimate 'rho (intraclass corr.)' the/(1+the);
run;

data skellam;	*---(4) Test of data in Skellam (J. Roy. Stat. Soc. B 10:257-261 [1948]);
input y @@;
n=3;
datalines;
   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
   0   0   0   0   0   0   0   0   0   0   0   0   0   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3
;
title 'Fitting distrete distributions to Skellam (1948), Ex. 4';
proc nlmixed data=skellam;	*---Assumes that counts/SU are in y (not x);
							*---Assumes n defines number of obs./SU;
    title2 'Binomial distribution';
    parameters lp = 2.0 ; 	*--Instead of directly estimating p, the LOGIT is directly modeled;
							*--Formulated as a logit linear predictor (lp ), for possible later treatment analysis;
							*--Give initial guess;
    p=1/(1+exp(-lp));		*--Probability of disease;
    
	model y ~ binomial(n,p);		*--Use the NLMIXED binomial(,) distribution;
	estimate 'p' 1/(1+exp(-lp));	*--Reformulate to get probability estimate (and se);
run;

*--The fitting of the beta-binomial using MLE;
proc nlmixed data=skellam;		*---Assumes that counts/SU are in y (not x);
								*---Assumes n defines number of obs./SU;
	title2 'Beta-binomial distribution';
    parameters lp = 2.0 the = 0.1 ; 	*--LOGIT linear predictor (lp) and theta (the);
										*--Give an initial guess;
    p=1/(1+exp(-lp));					*--Get p after first getting the logit;
    
	*--Following lines give the log-linear formulation for the BBD log-likelihood;
	*--The combinatorial and the Beta functions are all re-expresed as lgamma functions;
	
	C1 = (lgamma(n+1) -lgamma(y+1) - lgamma(n-y+1) );
	C2=lgamma((p/the)+((1-p)/the)) + lgamma((p/the)+y) + 
		lgamma(((1-p)/the) + n - y) -
		( lgamma(p/the) + lgamma((1-p)/the) + 
		lgamma((p/the) + ((1-p)/the) + n) );
	ll = C1 + C2;						*--Log-likelihood;

	model y ~ general(ll);				*--Tell NLMIXED that ll (defined above) is the user-specified log-likelihood;
	estimate 'p' 1/(1+exp(-lp));  	*--Get various parameterizations;
	estimate 'alpha' p/the;				
	estimate 'beta' (1-p)/the;
	estimate 'rho (intraclass corr.)' the/(1+the);
run;
