
/*
The GOF (Goodness of fit) macro will find the chi-squared (Pearson) goodness of fit for 
three discrete distributions (binomial, beta-binomial, and logistic-normal-binomial [LNB]).
A data set is required to use the macro that has y-values (usually counts) out of a fixed total value 
per sampling unit (n).
The data= , and n= , fields need to be specified by the user.
Also required are the parameter estimates for the distributions (previously obtained).
One can use the NLMIXED procedure to estimate parameters (see other SAS programs). 

Calculation of expected values is straight-forward for the binomial and beta-binomial, because
the (marginal) likelihood of the data is defined analytically. It is more challenging for the
LNB because the marginal likelihood requires numerical integration over the random 
sampling-unit effects. The macro uses quadrature to approximate the distribution to a 
high level of accuracy.

Frequencies are pooled, as needed, so that a proper chi-squared test is performed. 

			***	This macro is EXPERIMENTAL. ***

There are three options for entering parameter estimates from the binomial, betabinomial, 
and logistic-normal-binomial distributions.
1. If you enter values for pbinomial= , p= , theta= , eta= , and gamma= ,
		then results using the pbinomial= and the binomial distribution will be printed;
		results using the theta= and p= and beta-binomial distribution will be printed;
		and results using the eta= and gamma= and the logistic-normal-binomial distribution will be printed.

2. If you enter values for p= , theta= , eta= , and gamma= ,
		then results using the p= and the binomial distribution will be printed;
		results using the theta= and p= and beta-binomial distribution will be printed;
		and results using the eta= and gamma= and the logistic-normal-binomial distribution will be printed.

3. If you enter values for eta= , and gamma= ,
		then results using the eta= and gamma= and the logistic-normal-binomial distribution will be printed.

4. If you enter values for theta= , p= , and pbinomial= , 
        then results using the theta= and p= and beta-binomial distribution will be printed;
	    and results using the pbinomial= and the binomial distribution will be printed.

5. If you enter values for theta= and p= only, 
	    then results using the theta= and p= and beta-binomial distribution will be printed;
        and results using the p= and the binomial distribution will be printed;

6. If you enter a value for pbinomial= only, 
		then results using the pbinomial= and the binomial distribution will be printed.

(OPTIONAL)
nn= is the total number of sampling units. You may enter this manually or the program can find the value for you.
sumat= this is the value where different frequency classes will binned together. The default is 5. 
	That is, frequency classes will be binned together (put together)until the sum of their expected frequencies reaches 5. 
	Any frequency classes at the end that do not reach 5 by themselves are put with the previous bin that was
	greater than 5.
title= a title may be entered.
*/
/*
The output includes two files, Finaltable and Finalresults. Depending on the distributions you are using, the following results will be shown:
1. In the finaltable file, Count is the observed frequency, Binfreq is the expected binomila frequency, BBDfreq is the expected beta-binomial 
		frequency, and LNBfreq is the expected logistic-normal-binomial frequency.
2. In the finalresults file, 
		ChiSquarebin=Goodness-of-fit for binomial
		ChiSquareBBD=Goodness-of-fit for beta-binomial
		ChiSquareLNB=Goodness-of-fit for logistic-normal-binomial
		sumBinfreq, sumBBDfreq, sumLNBfreq = Sum of Binfreq, BBDfreq, and LNBfreq, respectively
		dfbin, dfBBD, dfLNB = degrees of freedom for binomial, beta-binomial, and logistic-normal-binomial goodness-of-fit test
		pbin, pBBD, pLNB = P value for binomial, beta-binomial, and logistic-normal-binomial goodness-of-fit test

A small P values suggests that the distribution did not fit the data. A large P value suggests that the distribution provides
a good fit to the data. 


Program written by Alissa Kriss, with input from L. V. Madden (Ohio State University).
Program supplied as a courtesy. We make no guarantees or warranties that it will
work correctly on all data sets. This macro is experimental.
*/

*---The GOF macro... The data files follow the macro.;

%macro GOF(
data= ,				/*name of input data set - NEEDS TO BE SPECIFIED*/
n= ,				/*observations per sampling unit - NEEDS TO BE SPECIFIED*/
nn= ,				/*total number of sampling units*/
pbinomial= ,		/*estimated parameter value from fitted binomial distribution*/
p= ,				/*estimated parameter value from fitted beta-binomial distribution 
					- will also give results for the Binomial distribution if pbinomial is not specified*/
theta= ,			/*estimated heterogeneity parameter value from fitted beta-binomial distribution*/
eta= ,				/*estimated linear predictor (LP) parameter value from fitted LNB distribution*/
gamma= ,			/*estimated heterogeneity parameter value from fitted LNB distribution*/
sumat= ,			/*value to bin expected frequencies at (optional, default of 5)*/
title= );			/*input a title if desired - no quotations*/

%let abort=0;
%if %length(&n)=0 %then %do; %put ERROR: Must specify n= variable.; %let abort=1; %goto DONE; %end;
%if &data=%str() %then %do; %put ERROR: Must specify input data set.; %let abort=1; %goto DONE; %end;
title8 "&title"; data aa; set &data; run;
%if %length(&nn)=0 %then %do; proc means data=aa noprint; var y; output out=tnew N(y)=Ny;
 data tnew; set tnew; call symput("nny",Ny); run; %let nn=&nny; %end;
%if %length(&sumat)=0 %then %let sumat=5; 
data aa; set aa; subject=_n_; run; proc freq data=aa noprint; tables y / out=_counts_; run; 
data l; do a=0 to &n; y=a; output l; end; run; data t; merge l _counts_; by y; run; 
data t2; set t; if count=. then count=0; run;

%if %length(&eta) ne 0 and %length(&gamma) ne 0 and %length(&theta) ne 0 and %length(&p) ne 0  
and %length (&pbinomial) %then %do;
title9 'Experimental discrete goodness of fit for Binomial, Betabinomial, and LNB Distributions';
data lnb1; set t2; run; data lnb1; sq2 = sqrt(2); s1=-5.38748; l1=-29.1319; s2=-4.60368; l2=-21.5444;
s3=-3.94476; l3=-16.0355; s4=-3.34785; l4=-11.7611; s5=-2.78881; l5=-8.3847; s6=-2.25497; l6=-5.7310;
s7=-1.73854; l7=-3.6965; s8=-1.23408; l8=-2.2162; s9=-0.73747; l9=-1.2494; s10=-0.24534; l10=-0.7717;
s11=0.24534;  l11=-0.7717; s12=0.73747;  l12=-1.2494; s13=1.23408;  l13=-2.2162; s14=1.73854;  l14=-3.6965;
s15=2.25497;  l15=-5.7310; s16=2.78881;  l16=-8.3847; s17=3.34785;  l17=-11.7611; s18=3.94476;  l18=-16.0355;
s19=4.60368;  l19=-21.5444; s20=5.38748;  l20=-29.1319;
array cr{20} l1 - l20; array sr{20} s1 - s20;
%do i = 0 %to &n; C1 = (lgamma(&n+1)-lgamma(&i+1)-lgamma(&n-&i+1)); aa = 0;
%do rr = 1 %to 20; lweight=cr{&rr}; weight=exp(lweight); sss=sr{&rr};
aa = aa  +   weight * ((exp(&eta + &gamma*sss*sq2))**&i) / ((1 + exp(&eta + &gamma*sss*sq2))**&n); laa = log(aa); %end;
D = C1 + log(aa); ProbLNB_ = exp(D); ProbLNB = ProbLNB_ * (3.1416**(-1/2)) ; LNBfreq = ProbLNB*&NN; output; %end; run;
data lnb1; set lnb1 (keep= LNBfreq); num=_n_; run; data t3; set t2;
alpha=&p/&theta; beta=(1-&p)/&theta; y2=(1-&p+(y*&theta))/(1+(y*&theta)); 
proc transpose data=t3 out=t4; var y2; run; 
data t5; set t4; array test(*) _numeric_; proby0=1; do i=1 to &n; proby0=proby0*test(i); end; drop i; run; 
data t5; set t5; subject=_n_; run; 
data t5; set t5; if subject=1 then call symput("y1",proby0); else stop; run; 
data t5; set t5 (keep=proby0); run; data t3a; merge t3 t5; run; data t3c; set t3a; L1=lag1(proby0); run; 
data t3d; set t3c; if a=1 then proby0=((&n+1-a)/a)*((alpha-1+a)/(&n+beta-a))*&y1; run; 
%do i=2 %to &n; data t3d; set t3d; L1=lag1(proby0); run; data t3d; set t3d; 
if a=&i then proby0=((&n+1-a)/a)*((alpha-1+a)/(&n+beta-a))*L1; run; %end;
data t3d; set t3d; BBDfreq=proby0*&nn; run; data t3d; set t3d; 
Binfreq=(fact(&n)/(fact(a)*fact(&n-a)))*(&pbinomial**a)*((1-&pbinomial)**(&n-a))*&nn; run; 
data finaltable; set t3d(keep=y count BBDfreq Binfreq); run; data finaltable; set finaltable; num=_n_; run; 
data finaltable; merge finaltable lnb1; by num; run; proc print data=finaltable (drop=num); run; 
proc sort data=finaltable out=b; by descending y; run; data b; set b; retain Elb Olb (0 0); retain Eb Ob (0 0); 
if Elb le &sumat then do; Elb=Elb+LNBfreq; Olb=Olb+Count; end; 
else do; Elb=LNBfreq; Olb=Count; end; if Eb le &sumat then do; Eb=Eb+BBDfreq; Ob=Ob+Count; end; else do; Eb=BBDfreq; 
Ob=Count; end; retain E O (0 0); if E le &sumat then do; E=E+Binfreq; O=O+Count; end; else do; E=Binfreq; O=Count; end; 
run; proc sort data=b out=zb; by y; run; data zb; set zb; retain Ezlb Ozlb (0 0); if Ezlb le &sumat then do; 
Ezlb=Ezlb+LNBfreq; Ozlb=Ozlb+Count; end; else do; Ezlb=LNBfreq; Ozlb=Count; end; retain Ezb Ozb (0 0); 
if Ezb le &sumat then do; Ezb=Ezb+BBDfreq; Ozb=Ozb+Count; end; else do; Ezb=BBDfreq; Ozb=Count; end; retain Ez Oz (0 0); 
if Ez le &sumat then do; Ez=Ez+Binfreq; Oz=Oz+Count; end;else do; Ez=Binfreq; Oz=Count; end;run; 
data zb; set zb; if Elb<&sumat then do; Elb=.; Olb=.; end; if Eb<&sumat then do; Eb=.; Ob=.; end; 
if E<&sumat then do; E=.; O=.; end; run;
data zb; set zb; if Elb=. then Ezlb=.; if Olb=. then Ozlb=.; if Eb=. then Ezb=.; if Ob=. then Ozb=.; if E=. then Ez=.; 
if O=. then Oz=.; run;data f; set zb; Enew=max(E,Ez); Onew=max(O,Oz); Enewb=max(Eb,Ezb); Onewb=max(Ob,Ozb); 
Enewlb=max(Elb,Ezlb); Onewlb=max(Olb,Ozlb);run; data f; set f; XX=((Onew-Enew)**2)/Enew; 
XXb=((Onewb-Enewb)**2)/Enewb; XXlb=((Onewlb-Enewlb)**2)/Enewlb; run;
proc means data=f n noprint; output out=f1 n(XX)=numXX n(XXb)=numXXb n(XXlb)=numXXlb sum(Binfreq)=sumBinfreq 
sum(BBDfreq)=sumBBDfreq sum(LNBfreq)=sumLNBfreq; var XX XXb XXlb Binfreq BBDfreq LNBfreq; run; 
data f2; set f1; dfbin=numXX-2; dfBBD=numXXb-3; dfLNB=numXXlb-3; run; 
proc univariate data=f noprint; output out=finalchi sum=ChiSquarebin ChiSquareBBD ChiSquareLNB; var XX XXb XXlb; run; 
data final; merge finalchi f2; run;
data finalresults; set final (Keep=ChiSquarebin ChiSquareBBD ChiSquareLNB dfbin dfBBD dfLNB sumBinfreq sumBBDfreq sumLNBfreq); run;
data finalresults; set finalresults; pbin=1-probchi(ChiSquarebin,dfbin); pBBD=1-probchi(ChiSquareBBD, dfBBD); 
pLNB=1-probchi(ChiSquareLNB, dfLNB); run;  
proc print data=finalresults; run; %end; 

%else %if %length(&eta) ne 0 and %length(&gamma) ne 0 and %length(&theta) ne 0 and %length(&p) ne 0 %then %do;
title9 'Experimental discrete goodness of fit for Binomial, Betabinomial, and LNB Distributions';
data lnb1; set t2; run; data lnb1; sq2 = sqrt(2); s1=-5.38748; l1=-29.1319; s2=-4.60368; l2=-21.5444;
s3=-3.94476; l3=-16.0355; s4=-3.34785; l4=-11.7611; s5=-2.78881; l5=-8.3847; s6=-2.25497; l6=-5.7310;
s7=-1.73854; l7=-3.6965; s8=-1.23408; l8=-2.2162; s9=-0.73747; l9=-1.2494; s10=-0.24534; l10=-0.7717;
s11=0.24534;  l11=-0.7717; s12=0.73747;  l12=-1.2494; s13=1.23408;  l13=-2.2162; s14=1.73854;  l14=-3.6965;
s15=2.25497;  l15=-5.7310; s16=2.78881;  l16=-8.3847; s17=3.34785;  l17=-11.7611; s18=3.94476;  l18=-16.0355;
s19=4.60368;  l19=-21.5444; s20=5.38748;  l20=-29.1319;
array cr{20} l1 - l20; array sr{20} s1 - s20;
%do i = 0 %to &n; C1 = (lgamma(&n+1) -lgamma(&i+1) - lgamma(&n-&i+1) ); aa = 0;
%do rr = 1 %to 20; lweight=cr{&rr}; weight=exp(lweight); sss=sr{&rr};
aa = aa  +   weight * ((exp(&eta + &gamma*sss*sq2))**&i) / ((1 + exp(&eta + &gamma*sss*sq2))**&n);
laa = log(aa); %end; D = C1 + log(aa); ProbLNB_ = exp(D); ProbLNB = ProbLNB_ * (3.1416**(-1/2)) ;
LNBfreq = ProbLNB*&NN; output; %end; run;
data lnb2; set lnb1 (keep= LNBfreq); run; data lnb2; set lnb2; num=_n_; run;
data t3; set t2;
alpha=&p/&theta; beta=(1-&p)/&theta; y2=(1-&p+(y*&theta))/(1+(y*&theta)); 
proc transpose data=t3 out=t4; var y2; run; 
data t5; set t4; array test(*) _numeric_; proby0=1; do i=1 to &n; proby0=proby0*test(i); end; drop i; run; 
data t5; set t5; subject=_n_; run; 
data t5; set t5; if subject=1 then call symput("y1",proby0); else stop; run; 
data t5; set t5 (keep=proby0); run; data t3a; merge t3 t5; run;
data t3c; set t3a; L1=lag1(proby0); run; 
data t3d; set t3c; if a=1 then proby0=((&n+1-a)/a)*((alpha-1+a)/(&n+beta-a))*&y1; run; 
%do i=2 %to &n; data t3d; set t3d; L1=lag1(proby0); run; 
data t3d; set t3d; if a=&i then proby0=((&n+1-a)/a)*((alpha-1+a)/(&n+beta-a))*L1; run;  %end;
data t3g; set t3d; BBDfreq=proby0*&nn; run; 
data t3h; set t3g; Binfreq=(fact(&n)/(fact(a)*fact(&n-a)))*(&p**a)*((1-&p)**(&n-a))*&nn; run; 
data finaltable; set t3h(keep=y count BBDfreq Binfreq); run; 
data finaltable; set finaltable; num=_n_; run; data finaltable; merge finaltable lnb2; by num; run; proc print data=finaltable (drop=num); run;
proc sort data=finaltable out=b; by descending y; run;
data lb2; set b; retain Elb Olb (0 0); if Elb le &sumat then do; Elb=Elb+LNBfreq; Olb=Olb+Count; end; else do; Elb=LNBfreq; Olb=Count; 
end; run;
data b2; set lb2; retain Eb Ob (0 0); if Eb le &sumat then do; Eb=Eb+BBDfreq; Ob=Ob+Count; end; else do; Eb=BBDfreq; Ob=Count; 
end; run; 
data c2; set b2; retain E O (0 0); if E le &sumat then do; E=E+Binfreq; O=O+Count; end; else do; E=Binfreq; O=Count; end; run;  
proc sort data=c2 out=zb; by y; run; 
data zlb; set zb; retain Ezlb Ozlb (0 0); if Ezlb le &sumat then do; Ezlb=Ezlb+LNBfreq; Ozlb=Ozlb+Count; end; else do; Ezlb=LNBfreq; Ozlb=Count; end; run; 
data zzb; set zlb; retain Ezb Ozb (0 0); if Ezb le &sumat then do; Ezb=Ezb+BBDfreq; Ozb=Ozb+Count; end; else do; Ezb=BBDfreq; Ozb=Count; end; run; 
data zz; set zzb; retain Ez Oz (0 0); if Ez le &sumat then do; Ez=Ez+Binfreq; Oz=Oz+Count; end; 
else do; Ez=Binfreq; Oz=Count; end; run; 
data zz1lb; set zz; if Elb<&sumat then do; Elb=.; Olb=.; end; run;
data zz1b; set zz1lb; if Eb<&sumat then do; Eb=.; Ob=.; end; run; 
data zz1; set zz1b; if E<&sumat then do; E=.; O=.; end; run; 
data zz2lb; set zz1; if Eb=. then Ezlb=.; if Olb=. then Ozlb=.; run; 
data zzb2; set zz2lb; if Eb=. then Ezb=.; if Ob=. then Ozb=.; run; 
data zz2; set zzb2; if E=. then Ez=.; if O=. then Oz=.; run; 
data zznew; set zz2; Enew=max(E,Ez); Onew=max(O,Oz); Enewb=max(Eb,Ezb); Onewb=max(Ob,Ozb); Enewlb=max(Elb,Ezlb); Onewlb=max(Olb,Ozlb);run; 
data f; set zznew; XX=((Onew-Enew)**2)/Enew; XXb=((Onewb-Enewb)**2)/Enewb; XXlb=((Onewlb-Enewlb)**2)/Enewlb; run;
proc means data=f n noprint; output out=f1 n(XX)=numXX n(XXb)=numXXb n(XXlb)=numXXlb sum(Binfreq)=sumBinfreq sum(BBDfreq)=sumBBDfreq sum(LNBfreq)=sumLNBfreq; 
var XX XXb XXlb Binfreq BBDfreq LNBfreq; run; 
data f2; set f1; dfbin=numXX-2; dfBBD=numXXb-3; dfLNB=numXXlb-3; run; 
proc univariate data=f noprint; output out=finalchi sum=ChiSquarebin ChiSquareBBD ChiSquareLNB; var XX XXb XXlb; run; 
data final; merge finalchi f2; run; 
data final1; set final (Keep=ChiSquarebin ChiSquareBBD ChiSquareLNB dfbin dfBBD dfLNB sumBinfreq sumBBDfreq sumLNBfreq); run;
data finalresults; set final1; pbin=1-probchi(ChiSquarebin,dfbin); pBBD=1-probchi(ChiSquareBBD, dfBBD); pLNB=1-probchi(ChiSquareLNB, dfLNB); run;
proc print data=finalresults; run;
%end; 

%else %if %length(&eta) ne 0 and %length(&gamma) ne 0 and %length(&pbinomial) ne 0 %then %do;
title9 'Experimental discrete goodness of fit for Binomial and LNB Distributions';
data lnb1; set t2; run; data lnb1; sq2 = sqrt(2); s1=-5.38748; l1=-29.1319; s2=-4.60368; l2=-21.5444;
s3=-3.94476; l3=-16.0355; s4=-3.34785; l4=-11.7611; s5=-2.78881; l5=-8.3847; s6=-2.25497; l6=-5.7310;
s7=-1.73854; l7=-3.6965; s8=-1.23408; l8=-2.2162; s9=-0.73747; l9=-1.2494; s10=-0.24534; l10=-0.7717;
s11=0.24534;  l11=-0.7717; s12=0.73747;  l12=-1.2494; s13=1.23408;  l13=-2.2162; s14=1.73854;  l14=-3.6965;
s15=2.25497;  l15=-5.7310; s16=2.78881;  l16=-8.3847; s17=3.34785;  l17=-11.7611; s18=3.94476;  l18=-16.0355;
s19=4.60368;  l19=-21.5444; s20=5.38748;  l20=-29.1319;
array cr{20} l1 - l20; array sr{20} s1 - s20;
%do i = 0 %to &n; C1 = (lgamma(&n+1) -lgamma(&i+1) - lgamma(&n-&i+1) ); aa = 0;
%do rr = 1 %to 20; lweight=cr{&rr}; weight=exp(lweight); sss=sr{&rr};
aa = aa  +   weight * ((exp(&eta + &gamma*sss*sq2))**&i) / ((1 + exp(&eta + &gamma*sss*sq2))**&n);
laa = log(aa); %end; D = C1 + log(aa); ProbLNB_ = exp(D); ProbLNB = ProbLNB_ * (3.1416**(-1/2)) ;
LNBfreq = ProbLNB*&NN; output; %end; run;
data lnb2; set lnb1 (keep= LNBfreq); run; data lnb2; set lnb2; num=_n_; run;
data t3hq; set t2; Binfreq=(fact(&n)/(fact(a)*fact(&n-a)))*(&pbinomial**a)*((1-&pbinomial)**(&n-a))*&nn; run; 
data finaltableq; set t3hq(keep=y count Binfreq); run; 
data finaltableq; set finaltableq; num=_n_; run; data finaltable; merge finaltableq lnb2; by num; run;proc print data=finaltable (drop=num); run;
proc sort data=finaltable out=bq; by descending y; run; 
data c2ql; set bq; retain Eql Oql (0 0); if Eql le &sumat then do; Eql=Eql+LNBfreq; Oql=Oql+Count; end; else do; 
Eql=LNBfreq; Oql=Count; end; run;
data c2q; set c2ql; retain Eq Oq (0 0); if Eq le &sumat then do; Eq=Eq+Binfreq; Oq=Oq+Count; end; else do; 
Eq=Binfreq; Oq=Count; end; run;  
proc sort data=c2q out=zbq; by y; run; 
data zzql; set zbq; retain Ezql Ozql (0 0); if Ezql le &sumat then do; Ezql=Ezql+LNBfreq; Ozql=Ozql+Count; end; 
else do; Ezql=LNBfreq; Ozql=Count; end; run; 
data zzq; set zzql; retain Ezq Ozq (0 0); if Ezq le &sumat then do; Ezq=Ezq+Binfreq; Ozq=Ozq+Count; end; 
else do; Ezq=Binfreq; Ozq=Count; end; run; 
data zz1ql; set zzq; if Eql<&sumat then do; Eql=.; Oql=.; end; run; 
data zz1q; set zz1ql; if Eq<&sumat then do; Eq=.; Oq=.; end; run; 
data zz2ql; set zz1q; if Eql=. then Ezql=.; if Oql=. then Ozql=.; run; 
data zz2q; set zz2ql; if Eq=. then Ezq=.; if Oq=. then Ozq=.; run; 
data zznewq; set zz2q; Enewq=max(Eq,Ezq); Onewq=max(Oq,Ozq); Enewql=max(Eql,Ezql); Onewql=max(Oql,Ozql);run; 
data fq; set zznewq; XXq=((Onewq-Enewq)**2)/Enewq; XXql=((Onewql-Enewql)**2)/Enewql;run; 
proc means data=fq n noprint; var XXq Binfreq XXql LNBfreq; output out=f1q n(XXq)=numXXq sum(Binfreq)=sumBinfreq 
n(XXql)=numXXql sum(LNBfreq)=sumLNBfreq; run; 
data f2q; set f1q; dfbin=numXXq-2; dfLNB=numXXql-3; run; 
proc univariate data=fq noprint; output out=finalchiq sum=ChiSquarebin sum=ChiSquareLNB; var XXq XXql; run; 
data finalq; merge finalchiq f2q; run; 
data final1q; set finalq (Keep=ChiSquarebin dfbin sumBinfreq ChiSquareLNB dfLNB sumLNBfreq); run; 
data finalresults; set final1q; pbin=1-probchi(ChiSquarebin,dfbin); pLNB=1-probchi(ChiSquareLNB,dfLNB); run; proc print data=finalresults; run; 
%end;

%else %if %length(&eta) ne 0 and %length(&gamma) ne 0 %then %do;
title9 'Experimental discrete goodness of fit for the LNB Distribution';
data lnb1; set t2; run; data lnb1; sq2 = sqrt(2); s1=-5.38748; l1=-29.1319; s2=-4.60368; l2=-21.5444;
s3=-3.94476; l3=-16.0355; s4=-3.34785; l4=-11.7611; s5=-2.78881; l5=-8.3847; s6=-2.25497; l6=-5.7310;
s7=-1.73854; l7=-3.6965; s8=-1.23408; l8=-2.2162; s9=-0.73747; l9=-1.2494; s10=-0.24534; l10=-0.7717;
s11=0.24534;  l11=-0.7717; s12=0.73747;  l12=-1.2494; s13=1.23408;  l13=-2.2162; s14=1.73854;  l14=-3.6965;
s15=2.25497;  l15=-5.7310; s16=2.78881;  l16=-8.3847; s17=3.34785;  l17=-11.7611; s18=3.94476;  l18=-16.0355;
s19=4.60368;  l19=-21.5444; s20=5.38748;  l20=-29.1319;
array cr{20} l1 - l20; array sr{20} s1 - s20;
%do i = 0 %to &n; C1 = (lgamma(&n+1) -lgamma(&i+1) - lgamma(&n-&i+1) ); aa = 0;
%do rr = 1 %to 20; lweight=cr{&rr}; weight=exp(lweight); sss=sr{&rr};
aa = aa  +   weight * ((exp(&eta + &gamma*sss*sq2))**&i) / ((1 + exp(&eta + &gamma*sss*sq2))**&n);
laa = log(aa); %end; D = C1 + log(aa); ProbLNB_ = exp(D); ProbLNB = ProbLNB_ * (3.1416**(-1/2));
LNBfreq = ProbLNB*&NN; output; %end; run;
data lnb2; set lnb1 (keep= LNBfreq); run; data finaltable; merge t2 lnb2; run; 
proc print data=finaltable (drop= a percent); run;
proc sort data=finaltable out=bq; by descending y; run; 
data c2q; set bq; retain Eq Oq (0 0); if Eq le &sumat then do; Eq=Eq+LNBfreq; Oq=Oq+Count; end; else do; 
Eq=LNBfreq; Oq=Count; end; run; proc sort data=c2q out=zbq; by y; run; 
data zzq; set zbq; retain Ezq Ozq (0 0); if Ezq le &sumat then do; Ezq=Ezq+LNBfreq; Ozq=Ozq+Count; end; 
else do; Ezq=LNBfreq; Ozq=Count; end; run; 
data zz1q; set zzq; if Eq<&sumat then do; Eq=.; Oq=.; end; run; 
data zz2q; set zz1q; if Eq=. then Ezq=.; if Oq=. then Ozq=.; run; 
data zznewq; set zz2q; Enewq=max(Eq,Ezq); Onewq=max(Oq,Ozq); run; 
data fq; set zznewq; XXq=((Onewq-Enewq)**2)/Enewq; run; 
proc means data=fq n noprint; var XXq LNBfreq; output out=f1q n(XXq)=numXXq sum(LNBfreq)=sumLNBfreq; run; 
data f2q; set f1q; dfLNB=numXXq-3; run; 
proc univariate data=fq noprint; output out=finalchiq sum=ChiSquareLNB; var XXq; run; 
data finalq; merge finalchiq f2q; run; 
data final1q; set finalq (Keep=ChiSquareLNB dfLNB sumLNBfreq); run; 
data finalresults; set final1q; pLNB=1-probchi(ChiSquareLNB,dfLNB); run; proc print data=finalresults; run; 
%end;

%else %if %length(&theta) ne 0 and %length(&p) ne 0 %then %do;
title9 'Experimental discrete goodness of fit for Binomial and Betabinomial Distributions';
data t3; set t2;
alpha=&p/&theta; beta=(1-&p)/&theta; y2=(1-&p+(y*&theta))/(1+(y*&theta)); 
proc transpose data=t3 out=t4; var y2; run; 
data t5; set t4; array test(*) _numeric_; proby0=1; do i=1 to &n; proby0=proby0*test(i); end; drop i; run; 
data t5; set t5; subject=_n_; run; 
data t5; set t5; if subject=1 then call symput("y1",proby0); else stop; run; 
data t5; set t5 (keep=proby0); run; data t3a; merge t3 t5; run;
data t3c; set t3a; L1=lag1(proby0); run; 
data t3d; set t3c; if a=1 then proby0=((&n+1-a)/a)*((alpha-1+a)/(&n+beta-a))*&y1; run; 
%do i=2 %to &n; data t3d; set t3d; L1=lag1(proby0); run; 
data t3d; set t3d; if a=&i then proby0=((&n+1-a)/a)*((alpha-1+a)/(&n+beta-a))*L1; run;  %end;
data t3g; set t3d; BBDfreq=proby0*&nn; run; 
data t3h; set t3g; Binfreq=(fact(&n)/(fact(a)*fact(&n-a)))*(&p**a)*((1-&p)**(&n-a))*&nn; run; 
data finaltable; set t3h(keep=y count BBDfreq Binfreq); run; proc print data=finaltable; run;
proc sort data=finaltable out=b; by descending y; run;
data b2; set b; retain Eb Ob (0 0); if Eb le &sumat then do; Eb=Eb+BBDfreq; Ob=Ob+Count; end; else do; Eb=BBDfreq; Ob=Count; 
end; run; 
data c2; set b2; retain E O (0 0); if E le &sumat then do; E=E+Binfreq; O=O+Count; end; else do; E=Binfreq; O=Count; end; run;  
proc sort data=c2 out=zb; by y; run; 
data zzb; set zb; retain Ezb Ozb (0 0); if Ezb le &sumat then do; Ezb=Ezb+BBDfreq; Ozb=Ozb+Count; end; else do; Ezb=BBDfreq; Ozb=Count; end; run; 
data zz; set zzb; retain Ez Oz (0 0); if Ez le &sumat then do; Ez=Ez+Binfreq; Oz=Oz+Count; end; 
else do; Ez=Binfreq; Oz=Count; end; run; 
data zz1b; set zz; if Eb<&sumat then do; Eb=.; Ob=.; end; run; 
data zz1; set zz1b; if E<&sumat then do; E=.; O=.; end; run; 
data zzb2; set zz1; if Eb=. then Ezb=.; if Ob=. then Ozb=.; run; 
data zz2; set zzb2; if E=. then Ez=.; if O=. then Oz=.; run; 
data zznew; set zz2; Enew=max(E,Ez); Onew=max(O,Oz); Enewb=max(Eb,Ezb); Onewb=max(Ob,Ozb); run; 
data f; set zznew; XX=((Onew-Enew)**2)/Enew; XXb=((Onewb-Enewb)**2)/Enewb; run; 
proc means data=f n noprint; output out=f1 n(XX)=numXX n(XXb)=numXXb sum(Binfreq)=sumBinfreq sum(BBDfreq)=sumBBDfreq; var XX XXb Binfreq BBDfreq; run; 
data f2; set f1; dfbin=numXX-2; dfBBD=numXXb-3; run; 
proc univariate data=f noprint; output out=finalchi sum=ChiSquarebin ChiSquareBBD; var XX XXb; run; 
data final; merge finalchi f2; run; 
data final1; set final (Keep=ChiSquarebin ChiSquareBBD dfbin dfBBD sumBinfreq sumBBDfreq); run;
data finalresults; set final1; pbinomial=1-probchi(ChiSquarebin,dfbin); pBBD=1-probchi(ChiSquareBBD, dfBBD); run;
proc print data=finalresults; run;
%end; 

%else %if %length(&pbinomial) ne 0 %then %do;
title9 'Experimental discrete goodness of fit for the Binomial Distribution';
data t3hq; set t2; Binfreq=(fact(&n)/(fact(a)*fact(&n-a)))*(&pbinomial**a)*((1-&pbinomial)**(&n-a))*&nn; run;
data finaltable; set t3hq(keep=y count Binfreq); run; proc print data=finaltable; run;
proc sort data=finaltable out=bq; by descending y; run; 
data c2q; set bq; retain Eq Oq (0 0); if Eq le &sumat then do; Eq=Eq+Binfreq; Oq=Oq+Count; end; else do; Eq=Binfreq; Oq=Count; end; run;  
proc sort data=c2q out=zbq; by y; run; 
data zzq; set zbq; retain Ezq Ozq (0 0); if Ezq le &sumat then do; Ezq=Ezq+Binfreq; Ozq=Ozq+Count; end; 
else do; Ezq=Binfreq; Ozq=Count; end; run; 
data zz1q; set zzq; if Eq<&sumat then do; Eq=.; Oq=.; end; run; 
data zz2q; set zz1q; if Eq=. then Ezq=.; if Oq=. then Ozq=.; run; 
data zznewq; set zz2q; Enewq=max(Eq,Ezq); Onewq=max(Oq,Ozq); run; 
data fq; set zznewq; XXq=((Onewq-Enewq)**2)/Enewq; run; 
proc means data=fq n noprint; var XXq Binfreq; output out=f1q n(XXq)=numXXq sum(Binfreq)=sumBinfreq; run; 
data f2q; set f1q; dfbin=numXXq-2; run; 
proc univariate data=fq noprint; output out=finalchiq sum=ChiSquarebin; var XXq; run; 
data finalq; merge finalchiq f2q; run; 
data final1q; set finalq (Keep=ChiSquarebin dfbin sumBinfreq); run; 
data finalresults; set final1q; pbin=1-probchi(ChiSquarebin,dfbin); run; proc print data=finalresults; run; %end;
%else %do; %put ERROR: Must specify eta= and gamma= and/or theta= and p= and/or pbinomial= estimates.; %let abort=1; %goto DONE; %end;

proc datasets; delete final1q finalq finalchiq f2q fq f1q zznewq zz2q zz1q zzq zbq c2q bq t3hq
final1 final finalchi f2 f f1 zznew zz2 zzb2 zz1 zz1b zz zzb zb c2 b2 finaltable t3g t3d t3h t3c t3a t5 t3 t4 t2 t l _counts_ lnb1
aa tnew b finalresults lnb2; run; quit;
%done: %if &abort %then %put ERROR: The Goodness of fit macro ended abnormally.;
title8; title9;
%mend GOF;
run;


data tobacco;
input y @@;	
datalines;
 	1    1    1    1    2    2    2    2    2    2    2    2    2    3    3
    3    3    3    3    3    3    4    4    4    4    4    4    4    4    4
    4    4    5    5    5    6    6    6    6    6    6    6    6    6    7
    7    7    7    7    7    7    7    8    8    8    8    8    8    9   10
   10   10   10   10   10   11   11   11   11   12   12   13   14   15   25
;

/*  Get the parameters from a fit of the distribution to the data (see other SAS files). */


%GOF(data=tobacco, 
n=40, 
nn= , 
pbinomial=.15, 
p=0.1563, 
theta=0.0486,
eta= , 
gamma= ,  
sumat= , 
title= tobacco virus example )   

/* By leaving eta and gamma blank (above), the LNB fit is not done. */

data skellam;	*Test of data in Skellam (J. Roy. Stat. Soc. B 10:257-261 [1948]);
input y @@;
datalines;
   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
   0   0   0   0   0   0   0   0   0   0   0   0   0   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1   1
   1   1   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2   2
   2   2   2   2   2   2   2   2   2   2   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3   3
   3   3   3   3   3   3   3   3   3   3   3   3   3   3
;
%GOF(data=skellam, 
n=3 , 
nn= , 
pbinomial=0.5807, 
p=0.5807, 
theta=0.0949,
eta= 0.349, 
gamma= 0.540, 
sumat= , 
title= data from Skellam paper )

data samita;
input y @@;
datalines;
0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0
0	0	0	0	0	0	0	0	0	0	0	0	0	1	1								
1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1
1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1	1
2	2	2	2	2	2	2	2	2	2	2	2	2	2	2	2	2	2	2	2	2	2	2
2	2	2	2	2	2	2	2	2	2	2	2	2	2	2								
3	3	3	3	3	3	3	3	3	3	3	3	3	3	3	3	3	3	3	3	3	3	3
4	4	4	4	4	4	4	4	4	4	6	7	5	5	5								
;
%GOF(data=samita, 
n=9 , 
nn= , 
pbinomial= 0.181, 
p= 0.181, 
theta= 0.053,
eta= -1.615, 
gamma= 0.585, 
sumat= , 
title= data from Samita PhD in Edinburgh)

